import greenfoot.*;
import java.util.List;

/**
 * TITLE:  Gravitational Universe Demo
 * AUTHOR:  danpost (greenfoot username)
 * DATE:  December 10, 2018
 * 
 * DESCRIPTION:  a world filled with gravitationally interacting particles.  Exhibits
 * both binary systems and massive "black" holes.
 * 
 * CONTROLS:  Use 'up' and 'down' arrow keys to adjust number of particles.
 */
public class MyWorld extends World
{
    int count = 5000; // 100 times the number of particles to be in world
    
    public MyWorld()
    {
        super(800, 600, 1);
        getBackground().setColor(Color.DARK_GRAY);
        getBackground().fill();
    }
    
    public void act()
    {
        // changing the number of particles
        if (Greenfoot.isKeyDown("up") && count < 10000) count++;
        if (Greenfoot.isKeyDown("down") && count > 200) count--;
        // listing particles in world
        List<Particle> particles = getObjects(Particle.class);
        // adjusting up number of particles in world
        if (particles.size() < count/100) addObject(new Particle(), rand(getWidth()), rand(getHeight()));
        // particle interactions
        for (int j=0; j<particles.size()-1; j++)
        {
            Particle a = particles.get(j);
            for (int i=j+1; i<particles.size(); i++)
            {
                Particle b = particles.get(i);
                interact(a, b);
            }
        }
        // adjusting down number of particles in world
        if (particles.size() > count/100) removeObject(particles.get(0));
        // displaying number of particles in world
        showText("Count: "+(count/100), 100, 20);
    }
    
    /**
     * interact method: applies gravitational force interaction between two particles
     */
    private void interact(Particle a, Particle b)
    {
        double dist = Math.hypot(a.xPos-b.xPos, a.yPos-b.yPos);
        // if (Math.abs(dist) > 255) return;
        double force = 4/(dist*dist)-2/(dist*dist*dist);
        double dir = Math.atan2(a.yPos-b.yPos, a.xPos-b.xPos)*180/Math.PI;
        b.addForce(force, dir);
        a.addForce(-force, dir);
    }
    
    /**
     * rand method: random number generator
     */
    public static int rand(int choiceCount)
    {
        return Greenfoot.getRandomNumber(choiceCount);
    }
}